@@ -39,6 +39,7 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstbytewriter.h>
+#include <gst/audio/audio.h>
 #include <gst/tag/tag.h>
 
 #include "gstoggmux.h"
@@ -149,14 +150,13 @@ gst_ogg_mux_class_init (GstOggMuxClass * klass)
   gobject_class->get_property = gst_ogg_mux_get_property;
   gobject_class->set_property = gst_ogg_mux_set_property;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&video_sink_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&audio_sink_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&subtitle_sink_factory));
+  gst_element_class_add_static_pad_template (gstelement_class, &src_factory);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &video_sink_factory);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &audio_sink_factory);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &subtitle_sink_factory);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Ogg muxer", "Codec/Muxer",
@@ -806,6 +806,7 @@ gst_ogg_mux_decorate_buffer (GstOggMux * ogg_mux, GstOggPadData * pad,
   GstClockTimeDiff diff;
   GstMapInfo map;
   ogg_packet packet;
+  gboolean end_clip = TRUE;
 
   /* ensure messing with metadata is ok */
   buf = gst_buffer_make_writable (buf);
@@ -853,20 +854,35 @@ gst_ogg_mux_decorate_buffer (GstOggMux * ogg_mux, GstOggPadData * pad,
   /* The last packet may have clipped samples. We need to test against
    * the segment to ensure we do not use a granpos that encompasses those.
    */
-  end_time =
-      gst_ogg_stream_granule_to_time (&pad->map, pad->next_granule + duration);
-  if (end_time > pad->segment.stop
-      && !GST_CLOCK_TIME_IS_VALID (gst_segment_to_running_time (&pad->segment,
-              GST_FORMAT_TIME, pad->segment.start + end_time))) {
-    gint64 actual_duration =
-        gst_util_uint64_scale_round (pad->segment.stop - time,
-        pad->map.granulerate_n,
-        GST_SECOND * pad->map.granulerate_d);
-    GST_INFO_OBJECT (ogg_mux,
-        "Got clipped last packet of duration %" G_GINT64_FORMAT " (%"
-        G_GINT64_FORMAT " clipped)", actual_duration,
-        duration - actual_duration);
-    duration = actual_duration;
+  if (pad->map.audio_clipping) {
+    GstAudioClippingMeta *cmeta = gst_buffer_get_audio_clipping_meta (buf);
+
+    g_assert (!cmeta || cmeta->format == GST_FORMAT_DEFAULT);
+    if (cmeta && cmeta->end && cmeta->end < duration) {
+      GST_DEBUG_OBJECT (pad->collect.pad,
+          "Clipping %" G_GUINT64_FORMAT " samples at the end", cmeta->end);
+      duration -= cmeta->end;
+      end_clip = FALSE;
+    }
+  }
+
+  if (end_clip) {
+    end_time =
+        gst_ogg_stream_granule_to_time (&pad->map,
+        pad->next_granule + duration);
+    if (end_time > pad->segment.stop
+        && !GST_CLOCK_TIME_IS_VALID (gst_segment_to_running_time (&pad->segment,
+                GST_FORMAT_TIME, pad->segment.start + end_time))) {
+      gint64 actual_duration =
+          gst_util_uint64_scale_round (pad->segment.stop - time,
+          pad->map.granulerate_n,
+          GST_SECOND * pad->map.granulerate_d);
+      GST_INFO_OBJECT (ogg_mux,
+          "Got clipped last packet of duration %" G_GINT64_FORMAT " (%"
+          G_GINT64_FORMAT " clipped)", actual_duration,
+          duration - actual_duration);
+      duration = actual_duration;
+    }
   }
 
   GST_LOG_OBJECT (pad->collect.pad, "buffer ts %" GST_TIME_FORMAT
@@ -887,9 +903,9 @@ gst_ogg_mux_decorate_buffer (GstOggMux * ogg_mux, GstOggPadData * pad,
   limit = MAX (limit, ogg_mux->max_tolerance);
 
   GST_LOG_OBJECT (pad->collect.pad, "expected granule %" G_GINT64_FORMAT " == "
-      "time %" GST_TIME_FORMAT " --> ts diff %" GST_TIME_FORMAT
+      "time %" GST_TIME_FORMAT " --> ts diff %" GST_STIME_FORMAT
       " < tolerance %" GST_TIME_FORMAT " (?)",
-      granule, GST_TIME_ARGS (next_time), GST_TIME_ARGS (ABS (diff)),
+      granule, GST_TIME_ARGS (next_time), GST_STIME_ARGS (diff),
       GST_TIME_ARGS (limit));
 
 resync:
