@@ -126,13 +126,12 @@ gst_opus_dec_class_init (GstOpusDecClass * klass)
   adclass->set_format = GST_DEBUG_FUNCPTR (gst_opus_dec_set_format);
   adclass->getcaps = GST_DEBUG_FUNCPTR (gst_opus_dec_getcaps);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&opus_dec_src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&opus_dec_sink_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &opus_dec_src_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &opus_dec_sink_factory);
   gst_element_class_set_static_metadata (element_class, "Opus audio decoder",
-      "Codec/Decoder/Audio",
-      "decode opus streams to audio",
+      "Codec/Decoder/Audio", "decode opus streams to audio",
       "Vincent Penquerc'h <vincent.penquerch@collabora.co.uk>");
   g_object_class_install_property (gobject_class, PROP_USE_INBAND_FEC,
       g_param_spec_boolean ("use-inband-fec", "Use in-band FEC",
@@ -167,6 +166,7 @@ gst_opus_dec_reset (GstOpusDec * dec)
   dec->sample_rate = 0;
   dec->n_channels = 0;
   dec->leftover_plc_duration = 0;
+  dec->last_known_buffer_duration = GST_CLOCK_TIME_NONE;
 }
 
 static void
@@ -238,22 +238,6 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
     GstCaps *constraint, *inter;
 
     constraint = gst_caps_from_string ("audio/x-raw");
-    if (dec->sample_rate == 0 || dec->sample_rate == 48000) {
-      gst_caps_set_simple (constraint, "rate", G_TYPE_INT, 48000, NULL);
-    } else {
-      GValue v = { 0 }
-      , l = {
-      0};
-      g_value_init (&l, GST_TYPE_LIST);
-      g_value_init (&v, G_TYPE_INT);
-      g_value_set_int (&v, dec->sample_rate);
-      gst_value_list_append_value (&l, &v);
-      g_value_set_int (&v, 48000);
-      gst_value_list_append_value (&l, &v);
-      gst_caps_set_value (constraint, "rate", &l);
-      g_value_unset (&v);
-      g_value_unset (&l);
-    }
     if (dec->n_channels <= 2) { /* including 0 */
       gst_caps_set_simple (constraint, "channels", GST_TYPE_INT_RANGE, 1, 2,
           NULL);
@@ -268,6 +252,7 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
     if (gst_caps_is_empty (inter)) {
       GST_DEBUG_OBJECT (dec, "Empty intersection, failed to negotiate");
       gst_caps_unref (inter);
+      gst_caps_unref (caps);
       return FALSE;
     }
 
@@ -284,6 +269,7 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
 
     dec->sample_rate = rate;
     dec->n_channels = channels;
+    gst_caps_unref (caps);
   }
 
   if (dec->n_channels == 0) {
@@ -331,7 +317,8 @@ gst_opus_dec_parse_header (GstOpusDec * dec, GstBuffer * buf)
   const GstAudioChannelPosition *posn = NULL;
 
   if (!gst_opus_header_is_id_header (buf)) {
-    GST_ERROR_OBJECT (dec, "Header is not an Opus ID header");
+    GST_ELEMENT_ERROR (dec, STREAM, FORMAT, (NULL),
+        ("Header is not an Opus ID header"));
     return GST_FLOW_ERROR;
   }
 
@@ -342,7 +329,8 @@ gst_opus_dec_parse_header (GstOpusDec * dec, GstBuffer * buf)
           &dec->n_streams,
           &dec->n_stereo_streams,
           dec->channel_mapping, &dec->pre_skip, &dec->r128_gain)) {
-    GST_ERROR_OBJECT (dec, "Failed to parse Opus ID header");
+    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL),
+        ("Failed to parse Opus ID header"));
     return GST_FLOW_ERROR;
   }
   dec->r128_gain_volume = gst_opus_dec_get_r128_volume (dec->r128_gain);
@@ -396,6 +384,66 @@ gst_opus_dec_parse_comments (GstOpusDec * dec, GstBuffer * buf)
   return GST_FLOW_OK;
 }
 
+/* adapted from ext/ogg/gstoggstream.c */
+static gint64
+packet_duration_opus (const unsigned char *data, size_t bytes)
+{
+  static const guint64 durations[32] = {
+    480, 960, 1920, 2880,       /* Silk NB */
+    480, 960, 1920, 2880,       /* Silk MB */
+    480, 960, 1920, 2880,       /* Silk WB */
+    480, 960,                   /* Hybrid SWB */
+    480, 960,                   /* Hybrid FB */
+    120, 240, 480, 960,         /* CELT NB */
+    120, 240, 480, 960,         /* CELT NB */
+    120, 240, 480, 960,         /* CELT NB */
+    120, 240, 480, 960,         /* CELT NB */
+  };
+
+  gint64 duration;
+  gint64 frame_duration;
+  gint nframes = 0;
+  guint8 toc;
+
+  if (bytes < 1)
+    return 0;
+
+  /* headers */
+  if (bytes >= 8 && !memcmp (data, "Opus", 4))
+    return 0;
+
+  toc = data[0];
+
+  frame_duration = durations[toc >> 3];
+  switch (toc & 3) {
+    case 0:
+      nframes = 1;
+      break;
+    case 1:
+      nframes = 2;
+      break;
+    case 2:
+      nframes = 2;
+      break;
+    case 3:
+      if (bytes < 2) {
+        GST_WARNING ("Code 3 Opus packet has less than 2 bytes");
+        return 0;
+      }
+      nframes = data[1] & 63;
+      break;
+  }
+
+  duration = nframes * frame_duration;
+  if (duration > 5760) {
+    GST_WARNING ("Opus packet duration > 120 ms, invalid");
+    return 0;
+  }
+  GST_LOG ("Opus packet: frame size %.1f ms, %d frames, duration %.1f ms",
+      frame_duration / 48.f, nframes, duration / 48.f);
+  return duration / 48.f * 1000000;
+}
+
 static GstFlowReturn
 opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
 {
@@ -492,6 +540,19 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
     GstClockTime aligned_missing_duration;
     GstClockTime missing_duration = GST_BUFFER_DURATION (bufd);
 
+    if (!GST_CLOCK_TIME_IS_VALID (missing_duration)) {
+      if (GST_CLOCK_TIME_IS_VALID (dec->last_known_buffer_duration)) {
+        missing_duration = dec->last_known_buffer_duration;
+        GST_WARNING_OBJECT (dec,
+            "Missing duration, using last duration %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (missing_duration));
+      } else {
+        GST_WARNING_OBJECT (dec,
+            "Missing buffer, but unknown duration, and no previously known duration, assuming 20 ms");
+        missing_duration = 20 * GST_MSECOND;
+      }
+    }
+
     GST_DEBUG_OBJECT (dec,
         "missing buffer, doing PLC duration %" GST_TIME_FORMAT
         " plus leftover %" GST_TIME_FORMAT, GST_TIME_ARGS (missing_duration),
@@ -544,6 +605,9 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
     goto buffer_failed;
   }
 
+  if (size > 0)
+    dec->last_known_buffer_duration = packet_duration_opus (data, size);
+
   gst_buffer_map (outbuf, &omap, GST_MAP_WRITE);
   out_data = (gint16 *) omap.data;
 
@@ -569,9 +633,12 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
     gst_buffer_unmap (buf, &map);
 
   if (n < 0) {
-    GST_ELEMENT_ERROR (dec, STREAM, DECODE, ("Decoding error: %d", n), (NULL));
+    GstFlowReturn ret = GST_FLOW_ERROR;
+
     gst_buffer_unref (outbuf);
-    return GST_FLOW_ERROR;
+    GST_AUDIO_DECODER_ERROR (dec, 1, STREAM, DECODE, (NULL),
+        ("Decoding error (%d): %s", n, opus_strerror (n)), ret);
+    return ret;
   }
   GST_DEBUG_OBJECT (dec, "decoded %d samples", n);
   gst_buffer_set_size (outbuf, n * 2 * dec->n_channels);
@@ -658,11 +725,13 @@ done:
   return res;
 
 creation_failed:
-  GST_ERROR_OBJECT (dec, "Failed to create Opus decoder: %d", err);
+  GST_ELEMENT_ERROR (dec, LIBRARY, INIT, ("Failed to create Opus decoder"),
+      ("Failed to create Opus decoder (%d): %s", err, opus_strerror (err)));
   return GST_FLOW_ERROR;
 
 buffer_failed:
-  GST_ERROR_OBJECT (dec, "Failed to create %u byte buffer", packet_size);
+  GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL),
+      ("Failed to create %u byte buffer", packet_size));
   return GST_FLOW_ERROR;
 }
 
@@ -862,45 +931,83 @@ gst_opus_dec_set_property (GObject * object, guint prop_id,
   }
 }
 
-GstCaps *
-gst_opus_dec_getcaps (GstAudioDecoder * dec, GstCaps * filter)
+/* caps must be writable */
+static void
+gst_opus_dec_caps_extend_channels_options (GstCaps * caps)
 {
-  GstCaps *caps;
   unsigned n;
   int channels;
 
-  if (filter) {
-    filter = gst_caps_copy (filter);
-    for (n = 0; n < gst_caps_get_size (filter); ++n) {
-      GstStructure *s = gst_caps_get_structure (filter, n);
-      if (gst_structure_get_int (s, "channels", &channels)) {
-        if (channels == 1 || channels == 2) {
-          GValue v = { 0 };
-          g_value_init (&v, GST_TYPE_INT_RANGE);
-          gst_value_set_int_range (&v, 1, 2);
-          gst_structure_set_value (s, "channels", &v);
-          g_value_unset (&v);
-        }
+  for (n = 0; n < gst_caps_get_size (caps); ++n) {
+    GstStructure *s = gst_caps_get_structure (caps, n);
+    if (gst_structure_get_int (s, "channels", &channels)) {
+      if (channels == 1 || channels == 2) {
+        GValue v = { 0 };
+        g_value_init (&v, GST_TYPE_INT_RANGE);
+        gst_value_set_int_range (&v, 1, 2);
+        gst_structure_set_value (s, "channels", &v);
+        g_value_unset (&v);
       }
     }
   }
-  caps = gst_audio_decoder_proxy_getcaps (dec, NULL, filter);
-  if (filter)
-    gst_caps_unref (filter);
+}
+
+static void
+gst_opus_dec_value_list_append_int (GValue * list, gint i)
+{
+  GValue v = { 0 };
+
+  g_value_init (&v, G_TYPE_INT);
+  g_value_set_int (&v, i);
+  gst_value_list_append_value (list, &v);
+  g_value_unset (&v);
+}
+
+static void
+gst_opus_dec_caps_extend_rate_options (GstCaps * caps)
+{
+  unsigned n;
+  GValue v = { 0 };
+
+  g_value_init (&v, GST_TYPE_LIST);
+  gst_opus_dec_value_list_append_int (&v, 48000);
+  gst_opus_dec_value_list_append_int (&v, 24000);
+  gst_opus_dec_value_list_append_int (&v, 16000);
+  gst_opus_dec_value_list_append_int (&v, 12000);
+  gst_opus_dec_value_list_append_int (&v, 8000);
+
+  for (n = 0; n < gst_caps_get_size (caps); ++n) {
+    GstStructure *s = gst_caps_get_structure (caps, n);
+
+    gst_structure_set_value (s, "rate", &v);
+  }
+  g_value_unset (&v);
+}
+
+GstCaps *
+gst_opus_dec_getcaps (GstAudioDecoder * dec, GstCaps * filter)
+{
+  GstCaps *caps, *proxy_filter = NULL, *ret;
+
+  if (filter) {
+    proxy_filter = gst_caps_copy (filter);
+    gst_opus_dec_caps_extend_channels_options (proxy_filter);
+    gst_opus_dec_caps_extend_rate_options (proxy_filter);
+  }
+  caps = gst_audio_decoder_proxy_getcaps (dec, NULL, proxy_filter);
+  if (proxy_filter)
+    gst_caps_unref (proxy_filter);
   if (caps) {
-    caps = gst_caps_copy (caps);
-    for (n = 0; n < gst_caps_get_size (caps); ++n) {
-      GstStructure *s = gst_caps_get_structure (caps, n);
-      if (gst_structure_get_int (s, "channels", &channels)) {
-        if (channels == 1 || channels == 2) {
-          GValue v = { 0 };
-          g_value_init (&v, GST_TYPE_INT_RANGE);
-          gst_value_set_int_range (&v, 1, 2);
-          gst_structure_set_value (s, "channels", &v);
-          g_value_unset (&v);
-        }
-      }
-    }
+    caps = gst_caps_make_writable (caps);
+    gst_opus_dec_caps_extend_channels_options (caps);
+    gst_opus_dec_caps_extend_rate_options (caps);
+  }
+
+  if (filter) {
+    ret = gst_caps_intersect (caps, filter);
+    gst_caps_unref (caps);
+  } else {
+    ret = caps;
   }
-  return caps;
+  return ret;
 }
