@@ -17,9 +17,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include <gst/gstcpuid.h>
-#include <stdint.h>
-
 static inline void
 inner_product_gint16_full_1_neon (gint16 * o, const gint16 * a,
     const gint16 * b, gint len, const gint16 * icoeff, gint bstride)
@@ -49,7 +46,7 @@ inner_product_gint16_full_1_neon (gint16 * o, const gint16 * a,
                   "      vld1.16 {d20}, [%[a]]!\n"
                   "      subs %[remainder], %[remainder], #4\n"
                   "      vmlal.s16 q0, d16, d20\n"
-                  "      bgt 3b\n"
+                  "      bne 3b\n"
                   "4:"
                   "      vadd.s32 d0, d0, d1\n"
                   "      vpadd.s32 d0, d0, d0\n"
@@ -100,7 +97,7 @@ inner_product_gint16_linear_1_neon (gint16 * o, const gint16 * a,
                   "      subs %[remainder], %[remainder], #4\n"
                   "      vmlal.s16 q0, d16, d24\n"
                   "      vmlal.s16 q1, d20, d24\n"
-                  "      bgt 3b\n"
+                  "      bne 3b\n"
                   "4:"
                   "      vld2.16 {d20[], d21[]}, [%[ic]]\n"
                   "      vshrn.s32 d0, q0, #15\n"
@@ -275,7 +272,7 @@ inner_product_gint32_full_1_neon (gint32 * o, const gint32 * a,
                   "      subs %[remainder], %[remainder], #4\n"
                   "      vmlal.s32 q0, d16, d20\n"
                   "      vmlal.s32 q0, d17, d21\n"
-                  "      bgt 3b\n"
+                  "      bne 3b\n"
                   "4:"
                   "      vadd.s64 d0, d0, d1\n"
                   "      vqrshrn.s64 d0, q0, #31\n"
@@ -490,7 +487,7 @@ inner_product_gfloat_full_1_neon (gfloat * o, const gfloat * a,
                   "      vld1.32 {q10}, [%[a]]!\n"
                   "      subs %[remainder], %[remainder], #4\n"
                   "      vmla.f32 q0, q6, q10\n"
-                  "      bgt 3b\n"
+                  "      bne 3b\n"
                   "4:"
                   "      vadd.f32 d0, d0, d1\n"
                   "      vpadd.f32 d0, d0, d0\n"
@@ -653,22 +650,22 @@ interpolate_gfloat_cubic_neon (gpointer op, const gpointer ap,
                     "q10", "q11", "q12", "q13", "q14", "q15", "memory");
 }
 
-MAKE_RESAMPLE_FUNC_STATIC (gint16, full, 1, neon);
-MAKE_RESAMPLE_FUNC_STATIC (gint16, linear, 1, neon);
-MAKE_RESAMPLE_FUNC_STATIC (gint16, cubic, 1, neon);
+MAKE_RESAMPLE_FUNC (gint16, full, 1, neon);
+MAKE_RESAMPLE_FUNC (gint16, linear, 1, neon);
+MAKE_RESAMPLE_FUNC (gint16, cubic, 1, neon);
 
-MAKE_RESAMPLE_FUNC_STATIC (gint32, full, 1, neon);
-MAKE_RESAMPLE_FUNC_STATIC (gint32, linear, 1, neon);
-MAKE_RESAMPLE_FUNC_STATIC (gint32, cubic, 1, neon);
+MAKE_RESAMPLE_FUNC (gint32, full, 1, neon);
+MAKE_RESAMPLE_FUNC (gint32, linear, 1, neon);
+MAKE_RESAMPLE_FUNC (gint32, cubic, 1, neon);
 
-MAKE_RESAMPLE_FUNC_STATIC (gfloat, full, 1, neon);
-MAKE_RESAMPLE_FUNC_STATIC (gfloat, linear, 1, neon);
-MAKE_RESAMPLE_FUNC_STATIC (gfloat, cubic, 1, neon);
+MAKE_RESAMPLE_FUNC (gfloat, full, 1, neon);
+MAKE_RESAMPLE_FUNC (gfloat, linear, 1, neon);
+MAKE_RESAMPLE_FUNC (gfloat, cubic, 1, neon);
 
-static inline void
-audio_resampler_check_neon (void)
+static void
+audio_resampler_check_neon (const gchar *option)
 {
-  if (gst_cpuid_supports_arm_neon()) {
+  if (!strcmp (option, "neon")) {
     GST_DEBUG ("enable NEON optimisations");
     resample_gint16_full_1 = resample_gint16_full_1_neon;
     resample_gint16_linear_1 = resample_gint16_linear_1_neon;
