@@ -343,6 +343,7 @@ video_format_is_packed (GstVideoFormat fmt)
     case GST_VIDEO_FORMAT_YVU9:
       return FALSE;
     case GST_VIDEO_FORMAT_IYU1:
+    case GST_VIDEO_FORMAT_IYU2:
     case GST_VIDEO_FORMAT_YUY2:
     case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
@@ -847,6 +848,62 @@ GST_START_TEST (test_parse_caps_multiview)
 
 GST_END_TEST;
 
+typedef struct
+{
+  const gchar *string_from;
+  const gchar *string_to;
+  const gchar *name;
+  GstVideoColorimetry color;
+} ColorimetryTest;
+
+#define MAKE_COLORIMETRY_TEST(s1,s2,n,r,m,t,p) { s1, s2, n,         \
+    { GST_VIDEO_COLOR_RANGE ##r, GST_VIDEO_COLOR_MATRIX_ ##m,       \
+    GST_VIDEO_TRANSFER_ ##t, GST_VIDEO_COLOR_PRIMARIES_ ##p } }
+
+GST_START_TEST (test_parse_colorimetry)
+{
+  ColorimetryTest tests[] = {
+    MAKE_COLORIMETRY_TEST ("bt601", "bt601", "bt601",
+        _16_235, BT601, BT709, SMPTE170M),
+    MAKE_COLORIMETRY_TEST ("2:4:5:4", "bt601", "bt601",
+        _16_235, BT601, BT709, SMPTE170M),
+    MAKE_COLORIMETRY_TEST ("bt709", "bt709", "bt709",
+        _16_235, BT709, BT709, BT709),
+    MAKE_COLORIMETRY_TEST ("smpte240m", "smpte240m", "smpte240m",
+        _16_235, SMPTE240M, SMPTE240M, SMPTE240M),
+    MAKE_COLORIMETRY_TEST ("sRGB", "sRGB", "sRGB",
+        _0_255, RGB, SRGB, BT709),
+    MAKE_COLORIMETRY_TEST ("bt2020" , "bt2020", "bt2020",
+        _16_235, BT2020, BT2020_12, BT2020),
+    MAKE_COLORIMETRY_TEST ("1:4:0:0", "1:4:0:0", NULL,
+        _0_255, BT601, UNKNOWN, UNKNOWN),
+  };
+  gint i;
+
+  for (i = 0; i < G_N_ELEMENTS (tests); i++) {
+    const ColorimetryTest *test = &tests[i];
+    GstVideoColorimetry color;
+    gchar *string;
+
+    fail_unless (gst_video_colorimetry_from_string (&color, test->string_from));
+    fail_unless_equals_int (color.range, test->color.range);
+    fail_unless_equals_int (color.matrix, test->color.matrix);
+    fail_unless_equals_int (color.transfer, test->color.transfer);
+    fail_unless_equals_int (color.primaries, test->color.primaries);
+
+    string = gst_video_colorimetry_to_string (&color);
+    fail_unless_equals_string (string, test->string_to);
+    g_free (string);
+
+    fail_unless (gst_video_colorimetry_is_equal (&color, &test->color));
+
+    if (test->name)
+      fail_unless (gst_video_colorimetry_matches (&color, test->name));
+  }
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_events)
 {
   GstEvent *e;
@@ -2515,16 +2572,16 @@ test_overlay_blend_rect_verify (gint x, gint y, gint width, gint height,
   for (; i < size - 4; i += 4) {
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
     /* B - G - R - A */
-    fail_unless_equals_int (data[i], 0x40);
-    fail_unless_equals_int (data[i + 1], 0x40);
-    fail_unless_equals_int (data[i + 2], 0x40);
-    fail_unless_equals_int (data[i + 3], 0x00);
+    fail_unless_equals_int (data[i], 0x80);
+    fail_unless_equals_int (data[i + 1], 0x80);
+    fail_unless_equals_int (data[i + 2], 0x80);
+    fail_unless_equals_int (data[i + 3], 0x80);
 #else
     /* A - R - G - B */
     fail_unless_equals_int (data[i], 0x00);
-    fail_unless_equals_int (data[i + 1], 0x40);
-    fail_unless_equals_int (data[i + 2], 0x40);
-    fail_unless_equals_int (data[i + 3], 0x40);
+    fail_unless_equals_int (data[i + 1], 0x80);
+    fail_unless_equals_int (data[i + 2], 0x80);
+    fail_unless_equals_int (data[i + 3], 0x80);
 #endif
     if ((i + 4) == (4 * (((((y > 0) ? (y + temp_height) : temp_height) -
                         1) * VIDEO_WIDTH) + ((x >
@@ -2635,6 +2692,60 @@ GST_START_TEST (test_overlay_blend)
 
 GST_END_TEST;
 
+GST_START_TEST (test_overlay_composition_over_transparency)
+{
+  GstVideoOverlayComposition *comp1;
+  GstVideoOverlayRectangle *rect1;
+  GstBuffer *pix1, *pix2;
+  GstVideoInfo vinfo;
+  guint8 *data;
+
+  GstVideoFrame video_frame;
+  guint fwidth = 200, height = 50, swidth = 100;
+
+  memset (&video_frame, 0, sizeof (GstVideoFrame));
+
+  pix1 = gst_buffer_new_and_alloc (fwidth * sizeof (guint32) * height);
+  gst_buffer_memset (pix1, 0, 0x00, gst_buffer_get_size (pix1));
+  gst_video_info_init (&vinfo);
+  gst_video_info_set_format (&vinfo, GST_VIDEO_OVERLAY_COMPOSITION_FORMAT_RGB,
+      fwidth, height);
+  gst_video_frame_map (&video_frame, &vinfo, pix1, GST_MAP_READWRITE);
+  gst_buffer_unref (pix1);
+
+  pix2 = gst_buffer_new_and_alloc (swidth * sizeof (guint32) * height);
+  gst_buffer_memset (pix2, 0, 0xFF, gst_buffer_get_size (pix2));
+  gst_buffer_add_video_meta (pix2, GST_VIDEO_FRAME_FLAG_NONE,
+      GST_VIDEO_OVERLAY_COMPOSITION_FORMAT_RGB, swidth, height);
+  rect1 = gst_video_overlay_rectangle_new_raw (pix2, swidth, 0,
+      swidth, height, GST_VIDEO_OVERLAY_FORMAT_FLAG_NONE);
+
+  comp1 = gst_video_overlay_composition_new (rect1);
+  fail_unless (gst_video_overlay_composition_blend (comp1, &video_frame));
+  gst_video_overlay_composition_unref (comp1);
+  gst_video_overlay_rectangle_unref (rect1);
+  gst_buffer_unref (pix2);
+
+  data = GST_VIDEO_FRAME_PLANE_DATA (&video_frame, 0);
+
+  fail_unless_equals_int (data[0], 0x00);
+  fail_unless_equals_int (data[1], 0x00);
+  fail_unless_equals_int (data[2], 0x00);
+  fail_unless_equals_int (data[3], 0x00);
+
+  data += swidth * sizeof (guint32);
+
+  fail_unless_equals_int (data[0], 0xFF);
+  fail_unless_equals_int (data[1], 0xFF);
+  fail_unless_equals_int (data[2], 0xFF);
+  fail_unless_equals_int (data[3], 0xFF);
+
+  gst_video_frame_unmap (&video_frame);
+}
+
+GST_END_TEST;
+
+
 static Suite *
 video_suite (void)
 {
@@ -2652,6 +2763,7 @@ video_suite (void)
   tcase_add_test (tc_chain, test_dar_calc);
   tcase_add_test (tc_chain, test_parse_caps_rgb);
   tcase_add_test (tc_chain, test_parse_caps_multiview);
+  tcase_add_test (tc_chain, test_parse_colorimetry);
   tcase_add_test (tc_chain, test_events);
   tcase_add_test (tc_chain, test_convert_frame);
   tcase_add_test (tc_chain, test_convert_frame_async);
@@ -2668,6 +2780,7 @@ video_suite (void)
   tcase_add_test (tc_chain, test_video_transfer);
   tcase_add_test (tc_chain, test_overlay_blend);
   tcase_add_test (tc_chain, test_video_center_rect);
+  tcase_add_test (tc_chain, test_overlay_composition_over_transparency);
 
   return s;
 }
